
debug = False
bd = None
bd_project = ''
bd_version = ''
bd_api = ''
bd_url = ''
bd_trustcert = False

version_match_reqd = False
ignore = False
no_ignore_test = False
no_ignore_synopsys = False
no_ignore_defaults = False
ignore_no_path_matches = False
review = False
report_file = ''